/* (The MIT License)
Copyright (c) 2006 Adam Bennett (cruxic@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
 */
package adamb.util;

/**
 A fixed size circular queue of bytes.
 */
public class CircularByteQueue
{
	private byte[] array;
	private int head;
	private int size;
	
	/**
	 @isFull true means the given buffer is full of valid data, in which case the
	 queue will initially be full.  Otherwise the queue will be empty.
	 */
	public CircularByteQueue(byte[] buffer, boolean isFull)
	{
		assert buffer.length > 0;
		array = buffer;
		head = 0;
		if (isFull)
			size = buffer.length;
		else
			size = 0;
	}
	
	public boolean equals(byte[] bytes)
	{
		if (bytes.length == size)
		{
			if (size > 0)
			{
				int n = array.length - head;
				int j = 0;
				for (int i = 0; i < n; i++)
				{
					if (array[head + i] == bytes[j])
						j++;
					else
						return false;
				}
				
				n = size - n;
				for (int i = 0; i < n; i++)
				{
					if (array[i] == bytes[j])
						j++;
					else
						return false;
				}
			}
			
			return true;
		}
		else
			return false;
	}
	
	public void put(byte b)
	{
		if (head + size < array.length)
		{
			array[head + size] = b;
			size++;
		}
		else
		{
			int tail = size - (array.length - head);
			array[tail] = b;
			
			//full?
			if (tail == head)
			{
				//move head
				head++;
				if (head == array.length)
					head = 0;
			}
			else
				size++;
		}
	}

}
