package us.lakora.brawl.rateconv;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import us.lakora.CustomScrollPane;
import us.lakora.DisplayLicense;

public class HelpDialog extends JDialog {
	private static final long serialVersionUID = 1L;

	private static final String STYLE = "<style>" +
				"p {" +
				"margin-bottom: 4px;" +
				"font-weight: normal;" +
				"font-size: 9px;" +
				"}" +
				".bb {" +
				"color: blue;" +
				"font-weight: bold;" +
				"}" +
				"</style>";
	
	public static final String HELP = "<html>" + STYLE +
			"<p><i>Options available in " + Shared.PROGRAM_TITLE + "</i></p>" +
			"<hr/><p><b>Looping options:</b></p>" +
			"<p><font class=\"bb\">Loop files start to end:</font> If enabled, Rate Converter will loop normally non-looping input<br/>" +
			"formats, such as FLAC and MP3. Either way, it will first look for a file called loop.txt,<br/>" +
			"which should contain information in the format:</p>" +
			"<p><font color=\"red\">loopStart loopEnd filename_here.[wav,flac,ogg]</font></p>" +
			"<p>For example:</p>" +
			"<p><font color=\"red\">86016 1743439 Lava Reef Zone Act 1.wav</font></p>" +
			"<p>If loop.txt has information for a particular file, that will override the value of this checkbox.</p>" +
			"<p>For looping inputs, such as BRSTM or .hps, the original loop points will be preserved.</p>" +
			"<hr/><p><b>Format options:</b></p>" +
			"<p><font class=\"bb\">Output as:</font> changes the output format. BRSTM (the format used in Super Smash Bros.<br/>" +
			"Brawl) is the default, but the program can also make output files in Ogg Vorbis, .logg<br/>" +
			"(looping Ogg), FLAC or WAV formats. If libmp3lame.dll is present in the " + Shared.SOX_DIR + " folder,<br/>" +
			"you can output to MP3 as well. You can get libmp3lame.dll from: <font color=\"red\">http://j.mp/mp3encoder</font></p>" +
			"<p><font class=\"bb\">Default rate:</font> sets the sample rate used for rendering VGM files. It also sets the rate<br/>" +
			"that files will be converted to if \"adjust to absolute rate\" is checked.</p>" +
			"<hr/><p><b>Filesize options:</b></p>" +
			"<p><font class=\"bb\">Convert to mono:</font> combines the channels into one, while keeping the overall volume.<br/>" +
			"For an uncompressed stereo file, converting to mono will cut the filesize in half.</p>" +
			"<p><font class=\"bb\">Abjust to absolute rate:</font> converts the selected files to the \"default\" sample rate,<br/>" +
			"unless their current rate is equal or less. (Raising the sample rate would increase<br/>" +
			"filesize but not quality.)</p>" +
			"<p><font class=\"bb\">Rate by factor of:</font> converts the selected files to a lower sample rate,<br/>" +
			"equal to the current rate times the number entered (must be between 0 and 1.)<br/>" +
			"For example, entering 0.5 in this field would cut the sample rate in half.</p>" +
			"<p><font class=\"bb\">Don't go below [number]:</font> don't change the sample rate of the source file if the new rate<br/>" +
			"would be below the given number. This can be used when adjusting the rate by a factor.</p>" +
			"<hr/><p><b>Volume options:</b></p>" +
			"<p><font class=\"bb\">Adjustment in dB:</font> Makes the audio louder or quieter by the given amount of decibels.</p>" +
			"<p><font class=\"bb\">Amplify by a factor of:</font> Takes an factor by which to amplify the audio. For example,<br/>" +
			"a value of 2.0 would double the amplitude while a value of 0.5 would cut it in half.</p>" +
			"</html>";
	
	public static String ABOUT = "<html>" +
				STYLE +
				"<p><b>About "  + Shared.PROGRAM_TITLE + "</b> - <span class=\"bb\">http://lakora.us/brawl/rateconv</span></p>" +
				"<p>This program is written in Java. It acts as a wrapper to other programs to produce<br/>" +
				"BRSTM files for Super Smash Bros. Brawl.</p>" +
				"<p>Note: any audio format supported by vgmstream or SoX is accepted as an input format.<br/>" +
				"For looping input tracks, if the output is BRSTM or .logg, the loop points will be preserved.</p>" +
				"</html>";
	
	public static final String CREDITS = "<html>" +
				STYLE +
				"<p><b>SoX</b> (http://sox.sf.net) is used to process input files in standard audio formats, such as WAV<br/>" +
				"and FLAC, and to execute operations such as changing the sample rate and amplifying audio.<br/>" +
				"SoX also uses libmad.dll (included - see COPYING.txt) to decode MP3 files.</p>" +
				"<p><b>test.exe</b> from vgmstream (http://vgmstream.sf.net) is used to process input files in formats<br/>" +
				"used by video games, such as BRSTM (Brawl's format or newer versions) and HPS.</p>" +
				"<p><b>dspadpcm.exe</b> is part of the Nintendo Wii SDK, and is used to create DSP files for Revolution B.<br/>" +
				"If anyone knows of another (preferably open-source) command-line tool that can do the<br/>" +
				"same thing, I'd love to hear about it.</p>" +
				"<p><b>Revolution B</b> (from http://hcs64.com/vgm_ripping.html) is used to create BRSTM files from<br/>" +
				"DSP files. I've ported the BRSTM-building to Java to make the program run slightly faster.</p>" +
				"<p><b>VGMPlay</b> (http://vgm.mdscene.net/forum/viewtopic.php?t=112) is the official VGM player<br/>" +
				"application. Its dump-to-WAV feature is used to convert VGM and VGZ files to WAV.</p>" +
				"<p><b>jvorbiscomment</b> (https://sites.google.com/site/cruxic/jvorbiscomment) is a Java library used to<br/>" +
				"write comments to Ogg files. It's used when using .logg as the output format.</html>";
	
	public HelpDialog() {
		
		setLayout(new BorderLayout());
		setModalityType(ModalityType.APPLICATION_MODAL);
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		
		Dimension standard = new Dimension(550, 300);
		JTabbedPane tabbed = new JTabbedPane();
		this.add(tabbed, BorderLayout.CENTER);
		tabbed.addTab("About", new CustomScrollPane(ABOUT, standard));
		tabbed.addTab("Help", new CustomScrollPane(HELP, standard));
		tabbed.addTab("Credits", new CustomScrollPane(CREDITS, standard));

		Dimension wide = new Dimension(600, 250);
		DisplayLicense dl;
		dl = DisplayLicense.newInstanceFromFile("COPYING.txt");
		dl.setPreferredSize(wide);
		tabbed.addTab("COPYING", dl);
		
		dl = DisplayLicense.newInstanceFromFile("Apache License.txt");
		dl.setPreferredSize(wide);
		tabbed.addTab("Apache License", dl);
		
		JTabbedPane libLicenses = new JTabbedPane();
		tabbed.addTab("COPYING for Included Software", libLicenses);
		libLicenses.addTab("SoX and libmad", DisplayLicense.newInstanceFromFile(
				"gpl-2.0.txt"));
		libLicenses.addTab("vgmstream", DisplayLicense.newInstanceFromFile(
				Shared.VGMSTREAM_DIR + File.separator + "COPYING"));
		libLicenses.addTab("libvorbis", DisplayLicense.newInstanceFromFile(
				Shared.VGMSTREAM_DIR + File.separator + "libvorbis-1.2.0.COPYING.txt"));
		libLicenses.addTab("libogg", DisplayLicense.newInstanceFromFile(
				Shared.VGMSTREAM_DIR + File.separator + "libogg-1.1.3.COPYING.txt"));
		libLicenses.addTab("mpg123", DisplayLicense.newInstanceFromFile(
				Shared.VGMSTREAM_DIR + File.separator + "mpg123-1.4.3.COPYING.txt"));
		libLicenses.addTab("g7221", DisplayLicense.newInstanceFromFile(
				Shared.VGMSTREAM_DIR + File.separator + "g7221-gen-lic.txt"));
		for (int i=0; i<6; i++) {
			libLicenses.getComponentAt(i).setPreferredSize(wide);
		}
		
		JPanel okPanel = new JPanel();
		JButton ok = new JButton("OK");
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
				dispose();
			}
		});
		okPanel.add(ok);
		this.add(okPanel, BorderLayout.SOUTH);
		
		pack();
		
	}
	
}
