package us.lakora.brawl.rateconv;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import us.lakora.brawl.rateconv.music.NonLoopingMusic;

/*****************************************************************************
 *   Copyright 2012 Isaac Schemm
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   
 *   You may obtain a copy of the Apache License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 ****************************************************************************/

public class LoopTxtReader {
	
	private ArrayList<String> filenames;
	private ArrayList<Integer> loopStarts;
	private ArrayList<Integer> loopEnds;

	/**
	 * Makes an empty LoopTxtReader. Its setLoops function will use the default it's given on all songs.
	 */
	public LoopTxtReader() {
		filenames = new ArrayList<String>();
		loopStarts = new ArrayList<Integer>();
		loopEnds = new ArrayList<Integer>();
	}
	
	public LoopTxtReader(String filename) throws IOException {
		this(new File(filename));
	}
	
	public LoopTxtReader(File f) throws IOException {
		filenames = new ArrayList<String>();
		loopStarts = new ArrayList<Integer>();
		loopEnds = new ArrayList<Integer>();
		FileReader fr = new FileReader(f);
		if (f.exists()) {
			BufferedReader br = new BufferedReader(fr);
			String data = "";
			String readIn = br.readLine();
			while (readIn != null) {
				data = data + readIn + "\n";
				readIn = br.readLine();
			}
			br.close();
			fr.close();
			data = data.replaceAll("\t+", " "); // Replace tabs with spaces
			data = data.replaceAll("  +", " "); // Replace more than one space with just one space
			data = data.replaceAll("\n \n", "\n\n"); // Clear lines with only tabs or spaces
			
			while (!data.equals("")) {
				String line = data.substring(0, data.indexOf("\n"));
				if (line.length() > 0 && line.charAt(0) != '#') {
					int loopStart = Integer.parseInt(line.substring(0, line.indexOf(" ")));
					line = line.substring(line.indexOf(" ")+1);
					int loopEnd = Integer.parseInt(line.substring(0, line.indexOf(" ")));
					line = line.substring(line.indexOf(" ")+1);
					String inputName = line;
					
					if (loopStart % 14336 != 0) {
						int difference = 14336 - (loopStart % 14336);
						loopStart += difference;
						loopEnd += difference;
					}
	
					filenames.add(inputName);
					loopStarts.add(loopStart);
					loopEnds.add(loopEnd);
				}
				
				data = data.substring(data.indexOf("\n")+1);
			}
		}
	}
	
	public void setLoops(NonLoopingMusic b, boolean loop_if_not_in_file) {
		setLoops(b, b.getName(), loop_if_not_in_file);
	}
	
	public void setLoops(NonLoopingMusic b, String original_filename, boolean loop_if_not_in_file) {
		int index = filenames.indexOf(original_filename);
		if ((index > -1) && (loopStarts.get(index) > -1)) {
			b.setLoop(loopStarts.get(index), loopEnds.get(index));
		} else {
			if (loop_if_not_in_file) {
				b.setLoop(0, -1); // Start loop at beginning
			} else {
				b.setLoop(-1, -1); // Negative number - don't loop
			}
		}
	}
	
	public int getLoopStart(String filename) {
		int index = filenames.indexOf(filename);
		int retVal;
		if (index > -1) {
			retVal = loopStarts.get(index);
		} else {
			retVal = -1;
		}
		return retVal;
	}
	
	public int getLoopEnd(String filename) {
		int index = filenames.indexOf(filename);
		int retVal;
		if (index > -1) {
			retVal = loopEnds.get(index);
		} else {
			retVal = -1;
		}
		return retVal;
	}

}
