package us.lakora.brawl.rateconv;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.*;
import javax.swing.border.Border;

import java.util.List;

public class OptionDialog extends JDialog {
	private static final long serialVersionUID = 1L;
	
//	public static void main(String[] args) {
//		OptionSet os = new OptionSet();
//		if (new OptionDialog(os).ask()) System.out.println(os);
//	}
	
	private OptionDialog dialog;
	private OptionSet opts;
	private boolean keepGoing;
	
	private JCheckBox mono;
	private JCheckBox loopWavBox;
	private JTextField defaultSampleRate;
	private JRadioButton noAmpRadio;
	private JRadioButton dbAmpRadio;
	private JTextField dbAmpEntry;
	private JRadioButton factorAmpRadio;
	private JTextField factorAmpEntry;
	private JComboBox outputExtEntry;
	private JCheckBox loopTrimBox;
	
	private JRadioButton noneRadio;
	private JRadioButton absoluteRadio;
	private JRadioButton relativeRadio;
	private JTextField relativeEntry;
	private JCheckBox minRateBox;
	private JTextField minRateEntry;
	
	public OptionDialog(final OptionSet opts) {
		dialog = this;
		this.opts = opts;
		keepGoing = false;
		
//		JMenuBar menu = new JMenuBar();
//		setJMenuBar(menu);
//		
//		JMenu helpmenu = new JMenu("Help");
//		menu.add(helpmenu);
//		
//		JMenuItem about = new JMenuItem("About");
//		about.addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent arg0) {
//				new AboutDialog().setVisible(true);
//			}
//		});
//		helpmenu.add(about);
//		
//		JMenuItem help = new JMenuItem("Help");
//		help.addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent arg0) {
//				new HelpDialog().showDialog(0);
//			}
//		});
//		helpmenu.add(help);
//		
//		JMenuItem license = new JMenuItem("License");
//		license.addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent arg0) {
//				DisplayLicense.readFileFromFS(dialog, "COPYING.txt");
//			}
//		});
//		helpmenu.add(license);
		
		setTitle(Shared.PROGRAM_TITLE);
		setLayout(new BorderLayout());
		setModalityType(ModalityType.APPLICATION_MODAL);
		
		JPanel main = new JPanel();
		main.setLayout(new BoxLayout(main, BoxLayout.X_AXIS));
		add(main, BorderLayout.CENTER);
		add(new ButtonPanel(), BorderLayout.EAST);
		
		JPanel left = new JPanel();
		left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
		left.setAlignmentY(0);
		left.setBorder(BorderFactory.createEmptyBorder(0, 4, 4, 8));
		JPanel right = new JPanel();
		right.setLayout(new BoxLayout(right, BoxLayout.Y_AXIS));
		right.setAlignmentY(0);
		right.setBorder(BorderFactory.createEmptyBorder(0, 8, 4, 4));
		main.add(left);
		main.add(right);
		
		new Header("Channels", left);
		
		mono = new JCheckBox("Convert to mono");
		left.add(mono);
		
		left.add(new HSeparator());
		
		new Header("Looping", left);
		
		loopWavBox = new JCheckBox("<html>" +
				"Loop a file from start to end<br/>" +
				"if it's not in loop.txt</html>", true);
		left.add(loopWavBox);
		left.add(new JLabel("(for formats that don't store"));
		left.add(new JLabel("loop information, such as MP3)"));
		
		left.add(new HSeparator());
		
		new Header("Sample Rate Adjustment", right);
		
		noneRadio = new JRadioButton("Don't adjust sample rate", true);
		right.add(noneRadio);
		
		absoluteRadio = new JRadioButton("<html>Adjust to absolute value<br/>" +
				"(uses &quot;default rate&quot; below)</html>");
		right.add(absoluteRadio);
		
		JPanel factorRatePanel = new HPanel(right);
		relativeRadio = new JRadioButton("Factor of: ");
		relativeRadio.addItemListener(new FactorAmpRadioListener());
		factorRatePanel.add(relativeRadio);
		relativeEntry = new STextField();
		factorRatePanel.add(relativeEntry);

		ButtonGroup rateGroup = new ButtonGroup();
		rateGroup.add(noneRadio);
		rateGroup.add(absoluteRadio);
		rateGroup.add(relativeRadio);
		
		right.add(new HSeparator());
		
		new Header("Amplitude (Volume) Adjustment", left);

		noAmpRadio = new JRadioButton("Don't adjust amplitude", true);
		left.add(noAmpRadio);
		
		JPanel dbAmpPanel = new HPanel(left);
		dbAmpRadio = new JRadioButton("Adjustment in dB: ");
		dbAmpPanel.add(dbAmpRadio);
		dbAmpEntry = new STextField();
		dbAmpPanel.add(dbAmpEntry);
		
		JPanel factorAmpPanel = new HPanel(left);
		factorAmpRadio = new JRadioButton("Factor of: ");
		factorAmpPanel.add(factorAmpRadio);
		factorAmpEntry = new STextField();
		factorAmpPanel.add(factorAmpEntry);

		ButtonGroup ampGroup = new ButtonGroup();
		ampGroup.add(noAmpRadio);
		ampGroup.add(dbAmpRadio);
		ampGroup.add(factorAmpRadio);
		
		new Header("Sample Rate", right);

		right.add(new JLabel("Default rate:"));
		right.add(new JLabel("(vgm/vgz rendered at this rate)"));
		
		JPanel defaultSampleRatePanel = new HPanel(right);
		defaultSampleRate = new STextField("44100");
		defaultSampleRatePanel.add(defaultSampleRate);
		defaultSampleRatePanel.add(new JLabel(" samples/sec"));
		
		right.add(new HSeparator());
		
		minRateBox = new JCheckBox("Don't go below:", true);
		right.add(minRateBox);
		JPanel minRatePanel = new HPanel(right);
		minRateEntry = new STextField("16000");
		minRatePanel.add(minRateEntry);
		minRatePanel.add(new JLabel(" samples/sec"));
		
		minRateBox.setEnabled(false);
		minRateEntry.setEnabled(false);
		
		right.add(new HSeparator());
		
		JPanel ofp = new JPanel();
		ofp.setLayout(new BoxLayout(ofp, BoxLayout.X_AXIS));
		ofp.setAlignmentX(0);
		new Header("Output as:  ", ofp);
		
		outputExtEntry = new JComboBox();
		List<OutputFormat> list = OutputFormat.list();
		for (int i=0; i<list.size(); i++) {
			outputExtEntry.addItem(list.get(i));
		}
		outputExtEntry.addItemListener(new OutputFormatListener());
		ofp.add(outputExtEntry);
		right.add(ofp);
		
		loopTrimBox = new JCheckBox("Trim to end of first loop");
		loopTrimBox.setEnabled(false);
		right.add(loopTrimBox);

		pack();
	}
	
	private class OutputFormatListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent arg0) {
			OutputFormat of = (OutputFormat)arg0.getItem();
			if (of.canLoop()) {
				loopTrimBox.setEnabled(false);
			} else {
				loopTrimBox.setEnabled(true);
			}
		}
	}
	
	private class FactorAmpRadioListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent arg0) {
			AbstractButton button = (AbstractButton)arg0.getItem();
			if (button.isSelected()) {
				minRateBox.setEnabled(true);
				minRateEntry.setEnabled(true);
			} else {
				minRateBox.setEnabled(false);
				minRateEntry.setEnabled(false);
			}
		}
	}
	
	private static class HPanel extends JPanel {
		private static final long serialVersionUID = 1L;

		public HPanel(JComponent addTo) {
			super();
			setAlignmentX(0);
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			addTo.add(this);
		}
	}
	
	private static class HSeparator extends JSeparator {
		private static final long serialVersionUID = 1L;
		
		public HSeparator() {
			super();
		}
	}
	
	private static class Header extends JLabel {
		private static final long serialVersionUID = 1L;
		
		private final static Border hBorder = BorderFactory.createEmptyBorder(0,0,8,0);
		
		public Header(String s, JComponent addTo) {
			super(s);
			super.setForeground(Color.blue);
			setBorder(hBorder);
			addTo.add(this);
		}
	}
	
	private static class STextField extends JTextField {
		private static final long serialVersionUID = 1L;
		
		private static Dimension SIZE = new Dimension(100, 20);
		
		public STextField() {
			super();
			setMaximumSize(SIZE);
		}
		
		public STextField(String s) {
			this();
			setText(s);
		}
	}
	
	private class ButtonPanel extends JPanel {
		private static final long serialVersionUID = 1L;
		
		public ButtonPanel() {
			super();
			GridLayout layout = new GridLayout(6, 1);
			layout.setHgap(4);
			layout.setVgap(4);
			setLayout(layout);
			setPreferredSize(new Dimension(96, 128));

			JButton ok = new JButton("OK");
			add(ok);
			ok.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					keepGoing = true;
					dialog.setVisible(false);
				}
			});
			JButton cancel = new JButton("Cancel");
			add(cancel);
			cancel.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					dialog.setVisible(false);
				}
			});
			
			add(new JLabel());

			JButton help = new JButton("Help");
			add(help);
			help.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					new HelpDialog().setVisible(true);
				}
			});

			add(new JLabel());
			add(new JLabel());

//			JButton license = new JButton("<html><center>License<br/>Information</center></html>");
//			add(license);
//			license.addActionListener(new ActionListener() {
//				public void actionPerformed(ActionEvent arg0) {
//					DisplayLicense.readFileFromFS(dialog, "COPYING.txt");
//				}
//			});
		}
	}

	public boolean getKeepGoing() {
		return keepGoing;
	}

	public boolean ask() {
		setVisible(true);
		dispose();
		if (keepGoing) {
			populateOptionSet();
			return true;
		} else {
			return false;
		}
	}
	
	private void populateOptionSet() {
		opts.setConvertToMono(mono.isSelected());
		
		try {
			opts.setDefaultRate(Integer.parseInt(defaultSampleRate.getText()));
		} catch (NumberFormatException e) {
			JOptionPane.showMessageDialog(null, defaultSampleRate.getText() + " is not a number.");
		}
		
		if (dbAmpRadio.isSelected()) {
			try {
				opts.setAmpType(OptionSet.DB_TYPE);
				opts.setAmpAmount(Double.parseDouble(dbAmpEntry.getText()));
			} catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(null, dbAmpEntry.getText() + " is not a number.");
				opts.setAmpType(OptionSet.NO_AMP);
			}
		} else if (factorAmpRadio.isSelected()) {
			try {
				opts.setAmpType(OptionSet.AMPLITUDE_TYPE);
				opts.setAmpAmount(Double.parseDouble(factorAmpEntry.getText()));
			} catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(null, factorAmpEntry.getText() + " is not a number.");
				opts.setAmpType(OptionSet.NO_AMP);
			}
		}
		opts.setLoopWav(loopWavBox.isSelected());
		
		try {
			int defaultRate = Integer.parseInt(defaultSampleRate.getText());
			if (defaultRate <= 0) throw new NumberFormatException();
			opts.setDefaultRate(defaultRate);
		} catch (NumberFormatException e) {
			Shared.errorDialog("The sample rate " + defaultSampleRate.getText() + " is not a positive integer.\n" +
					"Defaulting to 44100.");
			opts.setConvertRate(OptionSet.RATE_NO_CONVERSION);
		}
		
		if (absoluteRadio.isSelected()) {
			opts.setConvertRate(OptionSet.RATE_ABSOLUTE);
		} else if (relativeRadio.isSelected()) {
			try {
				double rateChange = Double.parseDouble(relativeEntry.getText());
				if ((rateChange <= 0.0) || (rateChange >= 1.0)) throw new NumberFormatException();
				opts.setRateFactor(rateChange);
				opts.setConvertRate(OptionSet.RATE_RELATIVE);
			} catch (NumberFormatException e) {
				Shared.errorDialog("The ratio " + relativeEntry.getText() + " is not a number between 0 and 1.\n" +
						"The sample rate will not be changed.");
				opts.setConvertRate(OptionSet.RATE_NO_CONVERSION);
			}
		} else {
			opts.setConvertRate(OptionSet.RATE_NO_CONVERSION);
		}
		
		if (relativeRadio.isSelected() && minRateBox.isSelected()) {
			try {
				opts.setMinimumRate(Integer.parseInt(minRateEntry.getText()));
			} catch (NumberFormatException e) {
				System.out.println(minRateEntry.getText() + " is not a number. Won't use a minimum rate.");
				opts.setMinimumRate(0);
			}
		}
		
		opts.setOutputFormat((OutputFormat)outputExtEntry.getSelectedItem());
	}

}
