package us.lakora.brawl.rateconv;

/*****************************************************************************
 *   Copyright 2012 Isaac Schemm
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   
 *   You may obtain a copy of the Apache License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 ****************************************************************************/

public class OptionSet {
	
	private boolean convertToMono;

	private int convertRate;
	public static final int RATE_ABSOLUTE = 1;
	public static final int RATE_RELATIVE = 2;
	public static final int RATE_NO_CONVERSION = 3;
	
//	/**
//	 * Will store a fraction if convertRate==RATE_RELATIVE or an integer if convertRate==RATE_ABSOLUTE.
//	 */
//	private double newRate;
	private double ampAmount;
	private double rateFactor;
	private int defaultRate;

	private int ampType;
	public static final int NO_AMP = -1;
	public static final int AMPLITUDE_TYPE = 0;
	public static final int DB_TYPE = 2;
	public static final int MAX_AMP_WITHOUT_CLIPPING = 3;

	/**
	 * Loop WAV/FLAC/OGG files that are not listed in loop.txt from start to end.
	 * If false, they will not loop at all.
	 */
	private boolean loopWav;
	private int minimumRate;

	private String loopTxt;
	
	private OutputFormat outputFormat;
	private boolean loopTrim;
	
	private boolean runVGMLoopCheck;
	
	public String toString() {
		StringBuffer sb = new StringBuffer("OptionSet: ");
		sb.append("default rendering rate: " + defaultRate + ", ");
		if (convertToMono) sb.append("to mono, ");
		if (convertRate == RATE_ABSOLUTE) sb.append("to rate " + defaultRate + ", ");
		if (convertRate == RATE_RELATIVE) sb.append("to " + (rateFactor * 100.0) + "% of old rate, ");
		if (ampType == AMPLITUDE_TYPE) {
			if (ampAmount != 1.0) sb.append("amplification " + ampAmount + ", ");
		} else if (ampType == DB_TYPE) {
			if (ampAmount != 0.0) sb.append("amplification " + ampAmount + " dB, ");			
		} else if (ampType == MAX_AMP_WITHOUT_CLIPPING) {
			sb.append("maximum amplification without clipping, ");
		}
		if (minimumRate > 0) sb.append("minimum rate " + minimumRate + ", ");
		if (loopWav) {
			sb.append("loop WAVs not in " + loopTxt + ", ");
		} else {
			sb.append("don't loop WAVs not in " + loopTxt + ", ");
		}
		sb.append("outputFormat = \"" + outputFormat + "\" (" + outputFormat.getExt() + ")");
		if (loopTrim) {
			sb.append(" (trim to end of loop), ");
		} else {
			sb.append(", ");
		}
		return sb.substring(0, sb.length()-2);
	}
	
	public OptionSet() {
		// Define variable defaults. Some of these, such as rateChange and newRateGlobal, will never be used unless they are user-defined.
		convertToMono = false;
		convertRate = RATE_NO_CONVERSION;
		defaultRate = 44100; // default should not be used
		rateFactor = 1.0; // default should not be used
		ampAmount = 1.0; // default should not be used except to check if one has been defined
		ampType = NO_AMP;
		loopWav = true;
		minimumRate = 0; // default should not be used except to check if one has been defined
		loopTxt = Shared.DEFAULT_LOOP_TXT;
		runVGMLoopCheck = false;
		outputFormat = null;
	}

	public int getConvertRate() {
		return convertRate;
	}

	public void setConvertRate(int convertRate) {
		this.convertRate = convertRate;
	}
	
	public double getRateFactor() {
		return rateFactor;
	}

	public void setRateFactor(double rateFactor) {
		this.rateFactor = rateFactor;
	}

	public int getDefaultRate() {
		return defaultRate;
	}

	public boolean isConvertToMono() {
		return convertToMono;
	}

	public void setConvertToMono(boolean convertToMono) {
		this.convertToMono = convertToMono;
	}

	public double getAmpAmount() {
		return ampAmount;
	}

	public void setAmpAmount(double ampAmount) {
		this.ampAmount = ampAmount;
	}

	public double getAmpType() {
		return ampType;
	}

	public void setAmpType(int ampType) {
		this.ampType = ampType;
	}

	public void setDefaultRate(int defaultRate) {
		this.defaultRate = defaultRate;
	}

	public boolean isLoopWav() {
		return loopWav;
	}

	public void setLoopWav(boolean loopWav) {
		this.loopWav = loopWav;
	}

	public int getMinimumRate() {
		return minimumRate;
	}

	public void setMinimumRate(int minimumRate) {
		this.minimumRate = minimumRate;
	}
	
	public String getLoopTxt() {
		return loopTxt;
	}

	public void setLoopTxt(String loopTxt) {
		this.loopTxt = loopTxt;
	}
	
	public boolean isRunVGMLoopCheck() {
		return runVGMLoopCheck;
	}
	
	public void setRunVGMLoopCheck(boolean b) {
		this.runVGMLoopCheck = b;
	}

	public OutputFormat getOutputFormat() {
		return outputFormat;
	}

	public void setOutputFormat(OutputFormat outputFormat) {
		this.outputFormat = outputFormat;
	}

	public boolean isLoopTrim() {
		return loopTrim;
	}

	public void setLoopTrim(boolean loopTrim) {
		this.loopTrim = loopTrim;
	}

}
