package us.lakora.brawl.rateconv;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Arrays;

import javax.swing.JOptionPane;

public class Shared {

	public static final String DEFAULT_LOOP_TXT = "loop.txt";
	
	public static final File OUTPUT_DIR = new File("output");
	private static final String TOOLS_DIR = "tools";
	public static final String SOX_DIR = TOOLS_DIR + File.separator + "sox";
	public static final String VGMSTREAM_DIR = TOOLS_DIR + File.separator + "vgmstream";
	public static final String DSPADPCM_DIR = TOOLS_DIR + File.separator + "dspadpcm";
	public static final String VGMPLAY_DIR = TOOLS_DIR + File.separator + "VGMPlay";
	public static final String TMP_DIR = "tmp";
	
	public static final String PROGRAM_TITLE="BRSTM Converter 3.1";
	
	public static final String HELPMESSAGE = PROGRAM_TITLE + "\n" +
			"Usage: java -jar \"BRSTM Converter.jar\" [options] [filenames]\n" +
			"\n" +
			"If you define any options, the option dialog will not appear. Similarily, if you define any filenames, the file selection dialog will not appear.\n" +
			"Available options:\n" +
			"-m: Convert the files to mono.\n" +
			"-r [number]: Convert the sample rate of the files. A fractional value (e.g. 0.5 or 0.75) will be interpreted as a factor by which to change the current rate; an integer (e.g. 32000) will be taken as a new rate to use for all output files.\n" +
			"-d [number]: The sample rate for rendering VGMs. If you use an absolute value for -r above, these two values will be the same and one will override the other.\n" +
			"-a [number]: Amplify the audio by the given factor. Using a factor of 2.0, for example, would double the volume.\n" +
			"-min [number]: Set a minimum sample rate. If the factor by which to lower the sample rate would lower it under this value, the audio will be converted to this rate instead.\n" +
			"-loop: Loop WAV, FLAC, and Ogg Vorbis input files (start to end, or as specified in loop.txt.) On by default.\n" +
			"-noloop: Don't loop WAV/FLAC/Ogg files.\n" +
			"-l [filename]: Specify the text file to read looping information from. The default is loop.txt. Implies -loop.\n" +
			"-noopts: Don't show the options dialog. Not needed if you define other options on the command line.";
	
	/**
	 * Displays an error dialog via {@link JOptionPane} with {@link Shared#PROGRAM_TITLE} as the title of the window.
	 */
	public static void errorDialog(String s) {
		JOptionPane.showMessageDialog(null, s, PROGRAM_TITLE, JOptionPane.ERROR_MESSAGE);
	}
	
//	/**
//	 * Checks (currently via {@link File#separatorChar}) whether the user is running Windows or not.
//	 * @return an empty string if running Windows; "wine " otherwise.
//	 */
//	private static boolean wineCheck() {
//		boolean toRun;
//		if (File.separatorChar == '\\') {
//			toRun = false;
//		} else {
//			toRun = true;
//		}
//		return toRun;
//	}
//	
//	/**
//	 * Adds the element "wine" to the beginning of a String array if the user is not running Windows.
//	 * @param the original String array.
//	 * @return the new array. If on Windows, it will be identical.
//	 */
//	public static String[] wineCheck(String[] cmdLine) {
//		String[] newArray;
//		if (wineCheck()) {
//			newArray = new String[cmdLine.length+1];
//			newArray[0] = "wine";
//			for (int i=0; i<cmdLine.length; i++) {
//				newArray[i+1] = cmdLine[i];
//			}
//		} else {
//			newArray = cmdLine.clone();
//		}
//		return newArray;
//	}
//	
//	/**
//	 * Adds the element "wine" to the beginning of a String array if the user is not running Windows.
//	 * @param a List of Strings that will be converted to an Array.
//	 */
//	public static String[] wineCheck(List<String> arguments) {
//		if (wineCheck()) {
//			arguments.add(0, "wine");
//		}
//		String[] orig = new String[arguments.size()];
//		return arguments.toArray(orig);
//	}
	
	public static void progOut(Process p, String s) {
		if (s.length() > 0) System.out.println(s);

		InputStream stderr = p.getErrorStream();
		BufferedReader cleaner = new BufferedReader(new InputStreamReader(stderr));
		String line;
		try {
			line = cleaner.readLine();
			while (line != null) {
				System.err.println(line);
				line = cleaner.readLine();
			}
		} catch (IOException e) {
			e.printStackTrace(); Shared.errorDialog(e.getMessage());
			System.exit(1);
		}
		InputStream stdout = p.getInputStream();
		cleaner = new BufferedReader(new InputStreamReader(stdout));
		try {
			line = cleaner.readLine();
			while (line != null) {
				System.out.println(line);
				line = cleaner.readLine();
			}
			cleaner.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static void errorOut(Process p) throws IOException {
		BufferedReader cleaner2 = new BufferedReader(new InputStreamReader(p.getErrorStream()));
		String s2 = cleaner2.readLine();
		while (s2 != null) {
			System.out.println(s2);
			s2 = cleaner2.readLine();
		}
		cleaner2.close();	
	}
	
	public static <T> T[] concat(T[]... arrays) {
		if (arrays.length == 1) {
			return arrays[0];
		} else if (arrays.length > 1) {
			T[][] allButLast = Arrays.copyOf(arrays, arrays.length-1);
			T[] first = concat(allButLast);
			T[] second = arrays[arrays.length-1];
			T[] result = Arrays.copyOf(first, first.length + second.length);
			System.arraycopy(second, 0, result, first.length, second.length);
			return result;
		} else {
			return null;
		}
	}

}
