package us.lakora.brawl.rateconv.music;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.List;

import us.lakora.brawl.rateconv.Shared;

/*****************************************************************************
 *   Copyright 2012 Isaac Schemm
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   
 *   You may obtain a copy of the Apache License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 ****************************************************************************/

public abstract class NonLoopingMusic extends Music {
	
	public NonLoopingMusic(File input) {
		super(input);
	}

	/**
	 * Manually changes the loop start point.
	 * @param loopStart Start of loop, in samples. If this is a negative number, looping will be disabled; if it is 0 or greater; looping will be enabled.
	 * @param loopEnd End of loop, in samples. If this is less than the song length and more than loopStart, it replaces the length of the song; the old/real length won't be saved.
	 */
	public void setLoop(int loopStart, int loopEnd) {
		super.loopStart = loopStart;
		if ((loopEnd < super.length) && (loopEnd > loopStart)) {
			super.length = loopEnd;
		}
	}
	
	public boolean wasBrstm() {
		return false;
	}

	public boolean canConvert() {
		return true;
	}
	
	protected void readValuesWithSox() throws MusicFileException, IOException, InterruptedException {
		if (temporaryWavFile == null) createTemporaryWavFile(null);
		Process check = null;
		try {
			String[] cmdLine = {Shared.SOX_DIR + File.separator + "sox.exe", "--info", temporaryWavFile.getCanonicalPath()};
			check = Runtime.getRuntime().exec(cmdLine);
			InputStream stdout = check.getInputStream();
			BufferedReader cleaner = new BufferedReader(new InputStreamReader(stdout));
			String line = cleaner.readLine();
			if (line == null) { // if sox could not read the file and therefore gave no output
				handleSoxError(check.getErrorStream());
			}
			while (line != null) {
				if (line.startsWith("Channels")) {
					super.channels = Integer.parseInt(line.substring(line.indexOf(':')+2));
				} else if (line.startsWith("Sample Rate")) {
					super.sampleRate = Integer.parseInt(line.substring(line.indexOf(':')+2));
				} else if (line.startsWith("Duration")) {
					List<String> splitLine = Arrays.asList(line.split(" "));
					int index = splitLine.indexOf("samples");
					super.length = Integer.parseInt(splitLine.get(index-1));
				}
				line = cleaner.readLine();
			}
			cleaner.close();
			check.waitFor();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private void handleSoxError(InputStream stderr) throws IOException, MusicFileException {
		BufferedReader errorCleaner = new BufferedReader(new InputStreamReader(stderr));
		String line = errorCleaner.readLine();
		while (line != null) {
			System.out.println(line);
			line = errorCleaner.readLine();
		}
		errorCleaner.close();
		throw new MusicFileException("SoX could not read the file " + this);
	}

}
