package us.lakora.brawl.rateconv.music;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

import us.lakora.brawl.rateconv.ProgressBox;
import us.lakora.brawl.rateconv.Shared;

/*****************************************************************************
 *   Copyright 2012 Isaac Schemm
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   
 *   You may obtain a copy of the Apache License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 ****************************************************************************/

/**
 * This class is not just for BRSTMs, but any format vgmstream (test.exe) can play, including (for example) HPS files.
 */
public class Vgmstream extends Music {

	public Vgmstream(File input) throws MusicFileException, IOException {
		super(input);

		try {
//			String exeOutput = "";
			String[] cmdLine = {Shared.VGMSTREAM_DIR + File.separator + "test.exe", "-m", input.getCanonicalPath()};
			Process check = Runtime.getRuntime().exec(cmdLine);
			check.waitFor();

			if (check.exitValue() != 0) {
				throw new MusicFileException("test.exe cannot read " + super.getCanonicalPath());
			}

			BufferedReader cleaner = new BufferedReader(new InputStreamReader(check.getInputStream()));
			String line = cleaner.readLine();
			while (line != null) {
				if (line.contains("channels:")) {
					int channels = Integer.parseInt(line.substring(line.indexOf(" ")+1));
					super.channels = channels;
				} else if (line.contains("sample rate")) {
					String s = line;
					s = s.substring(line.indexOf(" ")+1); // cut out "sample"
					s = s.substring(s.indexOf(" ")+1); // cut out "rate"
					s = s.substring(0, s.indexOf(" ")); // cut out all but the number
					int sampleRate = Integer.parseInt(s);
					super.sampleRate = sampleRate;
				} else if (line.contains("loop start:")) {
					String s = line;
					s = s.substring(line.indexOf(" ")+1); // cut out "loop"
					s = s.substring(s.indexOf(" ")+1); // cut out "start"
					s = s.substring(0, s.indexOf(" ")); // cut out all but the number
					int loopStart = Integer.parseInt(s);
					super.loopStart = loopStart;
				} else if (line.contains("stream total samples:")) {
					String s = line;
					s = s.substring(line.indexOf(" ")+1); // cut out "stream"
					s = s.substring(s.indexOf(" ")+1); // cut out "total"
					s = s.substring(s.indexOf(" ")+1); // cut out "samples"
					s = s.substring(0, s.indexOf(" ")); // cut out all but the number
					int length = Integer.parseInt(s);
					super.length = length;
				}
//				exeOutput = exeOutput + line + "\n";
				line = cleaner.readLine();
			}
			cleaner.close();

			if (super.isLooping()) {
				int offBy = loopStart % SAMPLES_PER_BLOCK;
				if (offBy != 0) {
					super.loopStart += (SAMPLES_PER_BLOCK - offBy);
					super.length += (SAMPLES_PER_BLOCK - offBy);
				}
			} else {
				super.loopStart = -1;
				// Length is already set
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
			Shared.errorDialog(e.getMessage());
		}
	}
	
	public void createTemporaryWavFile(ProgressBox progressBox) throws IOException, InterruptedException, MusicFileException {
		progressBox.changeStatus("Decoding " + input.getName() + " to WAV...");
		
		String loops = "2.0"; // always use 2 loops in case the loop point is changed through sample rate conversion.
		File wavFile = new File(Shared.TMP_DIR + File.separator + "in.wav");
		Process getWav;
		String[] cmdLine = {Shared.VGMSTREAM_DIR + File.separator + "test.exe", "-o", wavFile.getCanonicalPath(), "-l", loops, "-f", "0.0", super.getCanonicalPath()};
		getWav = Runtime.getRuntime().exec(cmdLine);
		getWav.waitFor();
		if (getWav.exitValue() != 0) {
			throw new MusicFileException("Failed to create WAV files for " + this.getCanonicalPath());
		}
		super.temporaryWavFile = wavFile;
		super.temporaryWavFile.deleteOnExit();
		
		progressBox.changeStatus("Converting " + input.getName() + "...");
	}

}
